// 29.10.23
//
// ESP8266      ADF4351
//------------------------
// GPIO 13       DATA
// GPIO 14       CLK
// GPIO 15       LE
// VCC 3.3V      CE
//------------------------
//              SH1107
//------------------------
// GPIO 12       SCL
// GPIO  2       SDA
//
// PLL Output 1967.768MHz
// PLL Level -4dBm
// PLL Reference clock 30.72MHz

#include <SPI.h>
#include <ArduinoOTA.h>
#include <ESP8266WiFi.h>
#include <U8g2lib.h>
#include <SoftwareSerial.h>


#define RX (5)  // U-Blox UART TX
#define TX (4)  // U-Blox UART RX

U8G2_SH1107_SEEED_128X128_F_SW_I2C u8g2(U8G2_R1, /* clock=*/12, /* data=*/2, /* reset=*/U8X8_PIN_NONE);
EspSoftwareSerial::UART gnssSerial;


byte data[] {
  0x00, 0x58, 0x00, 0x05,  //R5
  0x00, 0x95, 0x26, 0x24,  //R4
  0x00, 0x00, 0x04, 0xB3,  //R3
  0x18, 0x00, 0xEE, 0x42,  //R2
  0x00, 0x00, 0x14, 0x01,  //R1
  0x00, 0xC0, 0x06, 0x98   //R0
};

byte CFG_PRT[] {
  0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00, 0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, 0x80, 0x25,
  0x00, 0x00, 0x23, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0xBC, 0x89
};  // UBX-CFG-PRT (UBX, Baud 9600)

byte CFG_NAV5[] {
  0xB5, 0x62, 0x06, 0x24, 0x24, 0x00, 0xFF, 0xFF, 0x02, 0x03, 0x00, 0x00, 0x00, 0x00, 0x10, 0x27,
  0x00, 0x00, 0x05, 0x00, 0xFA, 0x00, 0xFA, 0x00, 0x64, 0x00, 0x2C, 0x01, 0x00, 0x3C, 0x00, 0x00,
  0x00, 0x00, 0xC8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x16, 0xA0
};  // UBX-CFG-NAV5 (Stationary, Auto2D/3D, UTC Automatic)

byte inputBuf[72];


const char* ssid = "NTP";           // SSID of your Wifi network
const char* password = "199alK03";  // Wi-Fi Password
int PinMUX = 16;
int PllState = 0;
int k = 0;
long lastTime = 0;
long phase = 0;
long freq = 0;

IPAddress ip(192, 168, 0, 13);  //static IP address
IPAddress ip_gate(192, 168, 0, 1);
IPAddress ip_subnet(255, 255, 255, 0);
IPAddress ip_dns1(8, 8, 8, 8);
IPAddress ip_dns2(8, 8, 4, 4);

void ssidConnect() {
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  WiFi.config(ip, ip_gate, ip_subnet, ip_dns1, ip_dns2);
  while (WiFi.status() != WL_CONNECTED) {
    delay(3000);
    ArduinoOTA.setHostname("UPConverter QO-100");
    ArduinoOTA.begin();
  }
}


void setup() {

  ssidConnect();
  k = 0;
  gnssSerial.begin(9600, EspSoftwareSerial::SWSERIAL_8N1, RX, TX);
  if (gnssSerial.available() > 0) {
    gnssSerial.write(CFG_PRT, sizeof(CFG_PRT));
    delay(250);
    gnssSerial.write(CFG_NAV5, sizeof(CFG_NAV5));
  }
   
//  Serial.begin(57600);
  pinMode(PinMUX, INPUT);
  pinMode(SS, OUTPUT);
  pinMode(SS, HIGH);
  SPI.beginTransaction(SPISettings(1000000, MSBFIRST, SPI_MODE0));
  SPI.begin();
  delay(50);

  for (int i = 0; i < sizeof(data); i += 4) {
    digitalWrite(SS, LOW);
    SPI.transfer(char(data[i]));
    SPI.transfer(char(data[i + 1]));
    SPI.transfer(char(data[i + 2]));
    SPI.transfer(char(data[i + 3]));
    SPI.endTransaction();
    digitalWrite(SS, HIGH);
  }

  u8g2.begin();
  u8g2.setBusClock(400000);  // Set speed I2C 400 kHz
  u8g2.setFont(u8g2_font_7x14B_tf);
  u8g2.clearBuffer();
}

void loop() {

  if (k == 0 && gnssSerial.available() >= 2) {
    inputBuf[0] = gnssSerial.read();
    if (inputBuf[0] == 0xB5) {
      inputBuf[1] = gnssSerial.read();
      if (inputBuf[0] == 0xB5 && inputBuf[1] == 0x62) {
        k = 2;
      }
    }
  }
  if (k != 0) {
    while (gnssSerial.available() > 0) {
      inputBuf[k] = gnssSerial.read();
      k++;
      if (k >= 68) {
        k = 0;

        //Serial.println("rcv:");
        //for (int i = 0; i < 68; ++i) {
        //  Serial.print(" 0x");
        //  Serial.print(inputBuf[i], HEX);
        //}
        //Serial.println();
        int32_t *a = (int32_t*)&inputBuf[58];
        freq = *a;
       // Serial.print("Freq offset: ");
       // Serial.print((*a) * 0.0039);
       // Serial.println();
        a = (int32_t*)&inputBuf[46];
        phase = *a;
       // Serial.print("Phase offset: ");
       // Serial.print(*a);
       // Serial.println();
        break;
      }
    }
  }

  if (millis() - lastTime > 1000) {
    lastTime = millis();             // Update the timer

    PllState = digitalRead(PinMUX);  // Read Pin PllLock

    if (PllState == HIGH) {
      u8g2.setDrawColor(0);
      u8g2.drawBox(73, 33, 69, 14);
      u8g2.setDrawColor(1);
      u8g2.setCursor(12, 47);
      u8g2.print("ADF4351 :Lock");
    } else {
      u8g2.setDrawColor(0);
      u8g2.drawBox(73, 33, 69, 14);
      u8g2.setDrawColor(1);
      u8g2.setCursor(12, 47);
      u8g2.print("ADF4351 :Unlock");
    }

    u8g2.setDrawColor(0);
    u8g2.drawBox(73, 50, 69, 14);
    u8g2.setDrawColor(1);
    u8g2.setCursor(12, 62);
    if ((freq * 0.0039) < -5 || (freq * 0.0039) > 5) {
      u8g2.print("Freq ppb: Err");
    } else
    {
      u8g2.print("Freq ppb:"); u8g2.print((freq * 0.0039), 1);
    }
    u8g2.setDrawColor(0);
    u8g2.drawBox(73, 66, 69, 14);
    u8g2.setDrawColor(1);
    u8g2.setCursor(12, 78);
    if (phase < -30 || phase > 30) {
      u8g2.print("Phase ns: Err");
    } else
    {
      u8g2.print("Phase ns:"); u8g2.print(phase);
    }
    u8g2.sendBuffer();
    }
  ArduinoOTA.handle();
}
